# coding=utf-8
import logging
import os
import time
import random
import traceback
import requests
import json
import hmac
import hashlib
import binascii
import base64

from tencentcloud.common import credential
from tencentcloud.common.profile.client_profile import ClientProfile
from tencentcloud.common.profile.http_profile import HttpProfile
from tencentcloud.ocr.v20181119 import ocr_client, models


class Ocr(object):
    def __init__(self, keys):
        self.keys = keys
        self.app_id = None
        self.secret_id = None
        self.secret_key = None
        self.result = None
        os.environ['NO_PROXY'] = 'api.youtu.qq.com'  # 防止代理影响结果
        if len(keys) == 0:
            raise NameError('ocr appid or secret_id or secret_key is None')

    def __app_sign(self):
        now = int(time.time())
        expired = now + 2592000
        rdm = random.randint(0, 999999999)
        plain_text = 'a={}&k={}&e={}&t={}&r={}&u=xx&f='.format(self.app_id, self.secret_id, expired, now, rdm)
        b = hmac.new(self.secret_key.encode(), plain_text.encode(), hashlib.sha1)
        s = binascii.unhexlify(b.hexdigest()) + plain_text.encode('ascii')
        signature = base64.b64encode(s).rstrip()  # 生成签名
        return signature

    def __get_headers(self):
        sign = self.__app_sign()
        headers = {'Authorization': sign, 'Content-Type': 'text/json'}
        return headers

    def get_result_path(self, image_path):
        if len(image_path) == 0:
            return {'errormsg': 'IMAGE_PATH_EMPTY'}

        filepath = os.path.abspath(image_path)
        if not os.path.exists(filepath):
            return {'errormsg': 'IMAGE_FILE_NOT_EXISTS'}

        out = open(filepath, 'rb').read()
        return self.get_result(out)

    def get_result(self, image):
        self.result = None
        image = base64.b64encode(image)
        image = image.rstrip().decode('utf-8')

        for key in self.keys:  # 使用多个优图账号尝试,防止某个账号频率限制
            if 'error' not in key:
                key['error'] = 0  # 初始化碰到限制的次数
            if key['error'] > 3:
                continue  # 经常遇到频率限制的账号不用了
            self.app_id = key['app_id']
            self.secret_id = key['secret_id']
            self.secret_key = key['secret_key']
            headers = self.__get_headers()
            url = 'http://api.youtu.qq.com/youtu/ocrapi/generalocr'
            data = {"app_id": key['app_id'], "session_id": '', "image": image}

            try:
                r = requests.post(url, headers=headers, data=json.dumps(data))
                if r.status_code == 200:
                    r.encoding = 'utf-8'
                    self.result = r.json()
                    break
                else:
                    key['error'] = key['error'] + 1
                    logging.info('keys: {}'.format(self.keys))
                    logging.error('ocr请求返回异常:code {}, app_id {}'.format(r.status_code, key['app_id']))
            except Exception as e:
                traceback.print_exc()
                logging.error('error: {}'.format(traceback.format_exc()))

        if self.result and 'items' in self.result:
            return self.result
        else:
            logging.info('result:{}'.format(self.result))
            raise NameError('OCR 请求异常')


class TencentCloudOcr(object):
    def __init__(self, secret_id, secret_key):
        cred = credential.Credential(secret_id, secret_key)
        httpProfile = HttpProfile()
        httpProfile.endpoint = "ocr.tencentcloudapi.com"

        clientProfile = ClientProfile()
        clientProfile.httpProfile = httpProfile
        self.client = ocr_client.OcrClient(cred, "ap-guangzhou", clientProfile)

    def get_text_base_info(self, image):
        """
        通过图片的内容，获取图片上的文字内容
        :param image:
        :return:
        """
        req = models.GeneralFastOCRRequest()
        req.ImageBase64 = base64.b64encode(image).decode('utf-8')
        resp = self.client.GeneralFastOCR(req)
        logging.info('resp: {}'.format(resp.to_json_string()))
        return resp

    def get_result(self, image):
        """
        将腾讯云结果封装为优图结果，方便统一使用
        :param image:
        :return:
        """
        items = []
        resp = self.get_text_base_info(image)
        for info in resp.TextDetections:
            x = info.Polygon[0].X
            y = info.Polygon[0].Y
            width = info.Polygon[1].X - info.Polygon[0].X
            height = info.Polygon[3].Y - info.Polygon[0].Y
            box = {'x': x, 'y': y, 'width': width, 'height': height}
            items.append({'itemstring': info.DetectedText, 'itemcoord': box})
        return {'items': items}



# coding=utf-8
import re
import sys
import subprocess
import logging
import platform
import traceback

from func_timeout import func_timeout, FunctionTimedOut


class Util(object):
    def __init__(self, sn):
        self.is_win = 'window' in platform.system().lower()
        self.is_wsl = 'linux' in platform.system().lower() and 'microsoft' in platform.release().lower()  # 判断当前是不是WSL环境
        self.is_py2 = sys.version_info < (3, 0)
        self.sn = sn
        self.adb_path = None
        self.debug = False

    @staticmethod
    def __get_cmd_process(arg):
        logging.debug(arg)
        p = subprocess.Popen(arg, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)  # 将错误信息也使用stdout输出
        return p

    @staticmethod
    def __get_cmd_out(process):
        out, err = process.communicate()
        if err.strip():
            logging.error('命令 {} 有错误输出:\n{}'.format(process.args, err))

        return out

    @staticmethod
    def __run_cmd(arg, is_wait=True, encoding='utf-8'):
        p = Util.__get_cmd_process(arg)
        if is_wait:
            out, err = p.communicate()
        else:
            return p  # 如果不等待，直接返回

        if err:
            logging.error('err: {}, arg: {}'.format(err.strip(), arg))

        if encoding:
            out = out.decode(encoding)
            err = err.decode(encoding)

        try:
            logging.debug('out[: 100]: {}'.format(out[: 100].strip()))
        except Exception as e:
            error_str = traceback.format_exc().strip()
            logging.debug('out log error: {}'.format(error_str))

        return out, err

    @staticmethod
    def cmd(arg, timeout=30, is_wait=True, encoding='utf-8'):
        """
        执行命令，并返回命令的输出,有超时可以设置
        :param arg:
        :param timeout:
        :param is_wait:
        :param encoding:
        :return:
        """
        try:
            return func_timeout(timeout, Util.__run_cmd, args=(arg, is_wait, encoding))
        except FunctionTimedOut:
            print('执行命令超时 {}s: {}'.format(timeout, arg))

    def adb(self, arg, timeout=30, encoding='utf-8'):
        self.adb_path = self.adb_path if self.adb_path and self.adb_path != 'adb' else 'adb'

        if not self.sn:
            self.sn = self.get_first_sn()

        arg = '{} -s {} {}'.format(self.adb_path, self.sn, arg)
        for _ in range(3):
            out, err = self.cmd(arg, timeout, encoding=encoding)

            if err:  # 错误处理
                is_device_not_found = 'device' in err and 'not found' in err
                is_device_offline = 'device offline' in err
                if is_device_not_found or is_device_offline:
                    self.connect_sn()  # 尝试重新连接网络设备
            else:  # 没有错误，返回命令的结果
                return out

    def shell(self, arg, timeout=30, encoding='utf-8'):
        arg = 'shell {}'.format(arg)
        return self.adb(arg, timeout, encoding=encoding)

    def connect_sn(self):
        if self.sn.count('.') != 3:
            return  # 非网络设备不处理
        self.cmd('adb disconnect {}'.format(self.sn))  # 首先断开连接，排除该 sn 当前是 offline 状态
        self.cmd('adb connect {}'.format(self.sn))

    def get_first_sn(self):
        sn_info = self.get_sn_info()
        for sn in sn_info:
            if sn_info[sn] == 'device':
                return sn
        raise NameError('没有可以使用的设备: {}'.format(sn_info))

    def get_sn_info(self):
        sn_info = {}
        out, err = self.cmd('adb devices')
        lines = re.split(r'[\r\n]+', out.strip())
        for line in lines[1:]:
            if not line.strip():
                continue
            sn, status = re.split(r'\s+', line, maxsplit=1)
            sn_info[sn] = status
        return sn_info



# coding=utf-8
from .get_ui import GetUI
from .util import Util
from .adb_ext import AdbExt
from .tango import Tango


class Device(GetUI):
    def __init__(self, sn=None):
        self.util = Util(sn)
        self.adb_ext = AdbExt(self.util)
        GetUI.__init__(self, self.adb_ext)



# coding=utf-8
import os
import re
import time
import base64
import logging
import tempfile


class AdbExt(object):
    def __init__(self, util):
        self.util = util
        self.is_helper_ready = False
        self.width, self.height = None, None
        self.dir_path = os.path.dirname(os.path.abspath(__file__))  # 当前文件所在的目录绝对路径
        self.temp_device_dir_path = '/data/local/tmp'

    def init_device_size(self):
        if self.width and self.height:
            return
        out = self.util.shell('wm size')  # out like 'Physical size: 1080x1920'
        out = re.findall(r'\d+', out)
        self.width = int(out[0])
        self.height = int(out[1])

    def dump(self):
        for i in range(5):
            xml_str = self.__dump_xml()
            if xml_str:
                return xml_str
            time.sleep(1)
        raise NameError('dump xml fail!')

    def __dump_xml(self):
        # 使用 helper 获取 xml
        xml_str = self.run_helper_cmd('layout')

        # 使用压缩模式
        if not xml_str:
            xml_str = self.util.adb('exec-out uiautomator dump --compressed /dev/tty', encoding='')

        # 使用非压缩模式
        if not xml_str:
            xml_str = self.util.adb('exec-out uiautomator dump /dev/tty', encoding='')

        if isinstance(xml_str, bytes):
            xml_str = xml_str.decode('utf-8')

        if 'hierarchy' in xml_str:
            start = xml_str.find('<hierarchy')
            end = xml_str.rfind('>') + 1
            xml_str = xml_str[start: end].strip()
            return xml_str

    def run_helper_cmd(self, cmd):
        """
        执行 helper 的命令，当前 helper 支持 dump xml 和 screenshot
        :param cmd:
        :return:
        """
        if not self.is_helper_ready:
            if 'adbui' not in self.util.shell('ls {}'.format(self.temp_device_dir_path)):
                helper_path = os.path.join(self.dir_path, 'static', 'adbui')
                self.push(helper_path, self.temp_device_dir_path)
            self.is_helper_ready = True
        arg = 'app_process -Djava.class.path=/data/local/tmp/adbui /data/local/tmp com.ysbing.yadb.Main -{}'.format(cmd)
        return self.util.shell(arg)

    def delete_from_device(self, path):
        self.util.shell('rm -rf {}'.format(path))

    def screenshot(self, pc_path=None):
        out = self.run_helper_cmd('screenshot')
        if len(out) > 50:
            out = base64.b64decode(out)
        else:  # helper 截图失败，使用 screencap 截图
            logging.warning('helper 截图失败')
            arg = 'adb -s {} exec-out screencap -p'.format(self.util.sn)
            out = self.util.cmd(arg, encoding=None)  # 这里是 png bytes string

        # 保存截图
        if pc_path:
            if self.util.is_py2:
                pc_path = pc_path.decode('utf-8')
            if os.path.exists(pc_path):  # 删除电脑文件
                os.remove(pc_path)
            with open(pc_path, 'wb') as f:
                f.write(out)
            return pc_path

        return out

    def pull(self, device_path=None, pc_path=None):
        return self.util.adb('pull "{}" "{}"'.format(device_path, pc_path))

    def push(self, pc_path=None, device_path=None):
        return self.util.adb('push "{}" "{}"'.format(pc_path, device_path))

    def click(self, x, y):
        self.util.shell('input tap {} {}'.format(x, y))

    def long_click(self, x, y, duration=''):
        """
        长按
        :param x: x 坐标
        :param y: y 坐标
        :param duration: 长按的时间（ms）
        :return:
        """
        self.util.shell('input touchscreen swipe {} {} {} {} {}'.format(x, y, x, y, duration))

    def start(self, pkg):
        """
        使用monkey，只需给出包名即可启动一个应用
        :param pkg:
        :return:
        """
        self.util.shell('monkey -p {} 1'.format(pkg))

    def stop(self, pkg):
        self.util.shell('am force-stop {}'.format(pkg))

    def input(self, text):
        self.util.shell('input text "{}"'.format(text.replace('&', '\&')))

    def back(self, times=1):
        while times:
            self.util.shell('input keyevent 4')
            times -= 1

    def home(self):
        self.util.shell('input keyevent 3')

    def enter(self, times=1):
        while times:
            self.util.shell('input keyevent 66')
            times -= 1

    def swipe(self, e1=None, e2=None, start_x=None, start_y=None, end_x=None, end_y=None, duration=" "):
        """
        滑动事件，Android 4.4以上可选duration(ms)
        usage: swipe(e1, e2)
               swipe(e1, end_x=200, end_y=500)
               swipe(start_x=0.5, start_y=0.5, e2)
        """
        self.init_device_size()
        if e1 is not None:
            start_x = e1[0]
            start_y = e1[1]
        if e2 is not None:
            end_x = e2[0]
            end_y = e2[1]
        if 0 < start_x < 1:
            start_x = start_x * self.width
        if 0 < start_y < 1:
            start_y = start_y * self.height
        if 0 < end_x < 1:
            end_x = end_x * self.width
        if 0 < end_y < 1:
            end_y = end_y * self.height

        self.util.shell('input swipe %s %s %s %s %s' % (str(start_x), str(start_y), str(end_x), str(end_y), str(duration)))

    def clear(self, pkg):
        """
        重置应用
        :param pkg:
        :return:
        """
        self.util.shell('pm clear {}'.format(pkg))

    def wake_up(self):
        """
        点亮屏幕
        :return:
        """
        self.util.shell('input keyevent KEYCODE_WAKEUP')

    def unlock(self):
        """
        解锁屏幕
        :return:
        """
        self.util.shell('input keyevent 82')

    def grant(self, pkg, permission):
        """
        给app赋权限，类似 adb shell pm grant [PACKAGE_NAME] android.permission.PACKAGE_USAGE_STATS
        :return:
        """
        self.util.shell('pm grant {} {}'.format(pkg, permission))

    def install(self, apk_path, with_g=True, with_r=False, user=None):
        """
        安装包
        :param apk_path:
        :param with_g: -g 在一些设备上可以自动授权，默认 true
        :param with_r: -r 覆盖安装，默认 false
        :param user:
        :return:
        """
        arg = 'install'
        if user:
            arg = arg + ' -user {}'.format(user)
        if with_g:
            arg = arg + ' -g'
        if with_r:
            arg = arg + ' -r'
        self.util.adb('{} "{}"'.format(arg, apk_path))

    def uninstall(self, pkg):
        """
        卸载包
        :param pkg:
        :return:
        """
        self.util.adb('uninstall {}'.format(pkg))

    def get_name(self, remove_blank=False):
        name = self.util.shell('getprop ro.config.marketing_name').strip()
        if not name:
            name = self.util.shell('getprop ro.product.nickname').strip()
        if remove_blank:
            name = name.replace(' ', '')
        return name

    def switch_user(self, user_id, wait_time=5):
        self.util.shell('am switch-user {}'.format(user_id))
        time.sleep(wait_time)

    def list_packages(self, system=False):
        """
        返回手机中安装的包
        :param system:  是否包含系统包
        :return:
        """
        with_system = '' if system else '-3'
        return self.util.shell('pm list packages {}'.format(with_system))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020/3/29 3:55 下午
# @Author  : tangonian
# @Site    : 
# @File    : tango.py
# @Software: PyCharm
import os
import logging
import time


class Tango:
    @staticmethod
    def set_log(log_path, level=logging.INFO, remove_exist=False):
        if remove_exist and os.path.isfile(log_path):
            os.remove(log_path)

        handler1 = logging.StreamHandler()
        handler2 = logging.FileHandler(filename=log_path, encoding='utf-8')

        format_str = '{}%(asctime)s-%(levelname)s-%(lineno)d:%(message)s{}'
        handler1.setFormatter(logging.Formatter(format_str.format('', '')))
        handler2.setFormatter(logging.Formatter(format_str.format('', '')))

        logging.root.level = level
        logging.root.handlers = [handler1, handler2]

    @staticmethod
    def get_time_str(fmt='%Y%m%d-%H%M%S'):
        return time.strftime(fmt, time.localtime())

    @staticmethod
    def list_to_chunks(lst, n):
        """Yield successive n-sized chunks from lst."""
        for i in range(0, len(lst), n):
            yield lst[i:i + n]


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import io
from setuptools import setup, find_packages

VERSION = '4.5.10'

with io.open('README.md', 'r', encoding='utf-8') as fp:
    long_description = fp.read()

requires = [
    'lxml',
    'requests',
    'func_timeout',
    'tencentcloud-sdk-python>=3.0.0'
]

setup(
    name='adbui',
    version=VERSION,
    description='adbui 所有的功能都是通过 adb 命令，adbui 的特色是可以通过 xpath，ocr 获取 ui 元素。',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Tango Nian',
    author_email='hao1032@gmail.com',
    url='https://github.com/hao1032/adbui',
    keywords='testing android uiautomator ocr minicap',
    install_requires=requires,
    packages=find_packages(),
    include_package_data=True,
    license='MIT',
    platforms='any',
    classifiers=(
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Testing'
    )
)


# coding=utf-8
import sys
import re
import logging
import traceback
from lxml import etree
from .ocr import Ocr, TencentCloudOcr
from lxml.etree import tostring

short_keys = {'id': 'resource-id', 'class_': 'class', 'klass': 'class', 'desc': 'content-desc'}


class GetUI(object):
    def __init__(self, adb_ext):
        self.adb_ext = adb_ext
        self.keys = None
        self.xml = None
        self.ocr = None
        self.init_ocr()
        self.image = None

    def init_ocr(self, app_id=None, secret_id=None, secret_key=None, keys=None,
                 ten_secret_id=None, ten_secret_key=None):
        """
        初始化ocr对象，当前支持腾讯云和优图的ocr服务，优先使用腾讯云的ocr服务，优图的已经不再维护
        :param app_id:
        :param secret_id:
        :param secret_key:
        :param keys:
        :param ten_secret_id: 腾讯云的 secret_id
        :param ten_secret_key: 腾讯云的 secret_key
        :return:
        """
        if ten_secret_id and ten_secret_key:  # 优先使用腾讯云的 ocr
            self.ocr = TencentCloudOcr(ten_secret_id, ten_secret_key)
            return

        if keys is None:
            keys = []
        if app_id is None and secret_id is None and secret_key is None:
            # 以下为测试账号，任何人可用，但是随时都会不可用，建议自行去腾讯优图申请专属账号
            app_id = '10126986'
            secret_id = 'AKIDT1Ws34B98MgtvmqRIC4oQr7CBzhEPvCL'
            secret_key = 'AAyb3KQL5d1DE4jIMF2f6PYWJvLaeXEk'
        keys.append({'app_id': app_id, 'secret_id': secret_id, 'secret_key': secret_key})
        self.ocr = Ocr(keys)

    def get_ui_by_attr(self, is_contains=True, is_update=True, **kwargs):
        uis = self.get_uis_by_attr(is_contains=is_contains, is_update=is_update, **kwargs)
        return uis[0] if uis else None

    def get_uis_by_attr(self, is_contains=True, is_update=True, **kwargs):
        """
        通过节点的属性获取节点
        :param is_contains: 是否使用模糊查找
        :param is_update:
        :param kwargs:
        :return: 
        """
        for key in list(kwargs):
            if key in short_keys:
                kwargs[short_keys[key]] = kwargs.pop(key)
        if is_contains:
            s = list(map(lambda x: "contains(@{}, '{}')".format(x, kwargs[x]), kwargs))
            xpath = './/*[{}]'.format(' and '.join(s))
        else:
            s = list(map(lambda key: "[@{}='{}']".format(key, kwargs[key]), kwargs))
            xpath = './/*{}'.format(''.join(s))
        uis = self.get_uis_by_xpath(xpath, is_update=is_update)
        return uis

    def get_ui_by_xpath(self, xpath, is_update=True):
        uis = self.get_uis_by_xpath(xpath, is_update)
        return uis[0] if uis else None

    def get_uis_by_xpath(self, xpath, is_update=True):
        """
        通过xpath查找节点
        :param xpath: 
        :param is_update: 
        :return: 
        """
        if is_update:
            xml_str = self.adb_ext.dump()  # 获取xml文件
            self.__init_xml(xml_str)

        xpath = xpath.decode('utf-8') if sys.version_info[0] < 3 else xpath
        elements = self.xml.xpath(xpath)
        return list(map(lambda x: self.get_ui_by_element(x), elements))

    def get_ui_by_element(self, element):
        bounds = element.get('bounds')
        x1, y1, x2, y2 = re.compile(r"-?\d+").findall(bounds)
        ui = UI(self.adb_ext, x1, y1, x2, y2)
        ui.element = element
        text = element.get('text')
        if not text:
            text = element.get('content-desc')
        ui.text = text.encode('utf-8') if self.adb_ext.util.is_py2 and not isinstance(text, str) else text
        return ui

    def get_ui_by_ocr(self, text, is_contains=True, is_update=True):
        uis = self.get_uis_by_ocr(text, is_contains, is_update)
        return uis[0] if uis else None

    def get_uis_by_ocr(self, text, is_contains=True, is_update=True):
        """
        通过ocr识别获取节点
        :param is_contains:
        :param text: 查找的文本
        :param is_update: 是否重新获取截图
        :return: 
        """
        if self.ocr is None:
            raise NameError('ocr 功能没有初始化.请到 adbui 页面查看如何使用。\nhttps://github.com/hao1032/adbui')
        if is_update:
            self.image = self.adb_ext.screenshot()  # 获取截图

        ocr_result = self.ocr.get_result(self.image)
        text = text.decode('utf-8') if self.adb_ext.util.is_py2 and isinstance(text, str) else text
        uis = []
        for item in ocr_result['items']:
            item_string = item['itemstring']
            item_string = item_string.decode('utf-8') if self.adb_ext.util.is_py2 and isinstance(item_string, str) else item_string

            if (is_contains and text in item_string) or (not is_contains and text == item_string):
                item_coord = item['itemcoord']
                ui = UI(self.adb_ext, item_coord['x'], item_coord['y'],
                        item_coord['x'] + item_coord['width'], item_coord['y'] + item_coord['height'])
                ui.text = item_string
                uis.append(ui)
        return uis

    def __init_xml(self, xml_str):
        parser = etree.XMLParser(huge_tree=True)
        self.xml = etree.fromstring(xml_str, parser=parser)
        for element in self.xml.findall('.//node'):
            element.tag = element.get('class').split('.')[-1].replace('$', '')  # 将每个node的name替换为class值，和uiautomator里显示的一致


class UI:
    def __init__(self, adb_ext, x1, y1, x2, y2):
        self.__adb_ext = adb_ext
        self.x1 = int(x1)  # 左上角 x
        self.y1 = int(y1)  # 左上角 y
        self.x2 = int(x2)  # 右下角 x
        self.y2 = int(y2)  # 右下角 y
        self.width = self.x2 - self.x1  # 元素宽
        self.height = self.y2 - self.y1  # 元素高
        self.x = self.x1 + int(self.width / 2)
        self.y = self.y1 + int(self.height / 2)
        self.text = None  # 元素文本
        self.element = None  # 元素对应的 lxml element，ocr无效

    def get_element_str(self):
        return tostring(self.element)

    def get_value(self, key):
        # 返回 lxml element 属性对应的值
        if key in short_keys:
            key = short_keys[key]
        return self.element.get(key)

    def click(self):
        # 点击元素的中心点
        self.__adb_ext.click(self.x, self.y)
